import { Component, Input, ViewChild, Renderer2, ViewEncapsulation, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

/**
 * @record
 */

/**
 * @record
 */

/**
 * @record
 */

/**
 * @record
 */

/** @enum {number} */
const Separator = {
    NA: 0,
    Big: 1,
    Small: 2,
};
Separator[Separator.NA] = "NA";
Separator[Separator.Big] = "Big";
Separator[Separator.Small] = "Small";

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

const Config = {
    WIDTH: 200,
    ARC_STROKE: 5,
    ARROW_Y: 22.5,
    ARROW_WIDTH: 4,
    ARROW_PIN_RAD: 8,
    SL_NORM: 3,
    SL_MID_SEP: 7,
    SL_SEP: 10,
    SL_WIDTH: 2,
    TXT_MARGIN: 10,
    LIGHT_Y: 55,
    LIGHT_RADIUS: 10,
    S_FAC_Y: 80,
    DIGITAL_Y: 145,
    UNIT_Y: 155,
    MAX_PURE_SCALE_VAL: 1000,
    INIT_LINE_FREQ: 2,
    DEF_START: 225,
    DEF_END: 135
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const showError = (text) => {
    console.error(`GaugeComponent: ${text}`);
};
const validate = (props) => {
    if (!props.max) {
        showError('The maximal value is not set.');
    }
    if (!(0 <= props.start && props.start <= 359) || !(0 <= props.end && props.end <= 359)) {
        showError('The end and start must be between 0 and 359 degrees.');
    }
    if (props.light && props.light > props.max) {
        showError('The red light trigger value cannot be greater than the max value of the gauge.');
    }
    if (props.factor && props.factor >= props.max) {
        showError('The factor cannot be greater than or equal to the max value.');
    }
    if (props.sectors) {
        props.sectors.forEach((s) => {
            if (s.from > props.max || s.to > props.max) {
                showError('The sector bounds cannot be greater than the max value.');
            }
            if (s.from >= s.to) {
                showError('The lower bound of the sector cannot be greater than or equal to the upper one.');
            }
            if (!s.color) {
                showError(`Sector[${s.from}, ${s.to}] color is empty.`);
            }
        });
    }
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GaugeComponent {
    /**
     * @param {?} _renderer
     */
    constructor(_renderer) {
        this._renderer = _renderer;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set input(val) {
        this._input = val;
        this._updateArrowPos(val);
    }
    /**
     * @return {?}
     */
    get input() {
        return this._input;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set max(val) {
        if (this._max) {
            this._max = val;
            this._initialize();
        }
        this._max = val;
    }
    /**
     * @return {?}
     */
    get max() {
        return this._max;
    }
    /**
     * @return {?}
     */
    get arc() {
        return this._arc(0, this._end);
    }
    /**
     * @return {?}
     */
    get gaugeRotationAngle() {
        return this._end - this.end;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.config = Object.assign(Config, this.config);
        if (!this.start) {
            this.start = this.config.DEF_START;
        }
        if (!this.end) {
            this.end = this.config.DEF_END;
        }
        validate(this);
        const /** @type {?} */ width = this.config.WIDTH + this.config.ARC_STROKE;
        this.viewBox = `0 0 ${width} ${width}`;
        this.radius = this.config.WIDTH / 2;
        this.center = width / 2;
        this._end = this.end;
        if (this.start > this.end) {
            this._end += (360 - this.start);
        }
        else {
            this._end -= this.start;
        }
        this._initialize();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this._rotateGauge();
    }
    /**
     * Initialize gauge.
     * @return {?}
     */
    _initialize() {
        this.scaleLines = [];
        this.scaleValues = [];
        this._calculateSectors();
        this._updateArrowPos(this._input);
        this.scaleFactor = this.factor || this._determineScaleFactor();
        this._createScale();
    }
    /**
     * Calculate arc.
     * @param {?} start
     * @param {?} end
     * @return {?}
     */
    _arc(start, end) {
        const /** @type {?} */ largeArc = end - start <= 180 ? 0 : 1;
        const /** @type {?} */ startCoor = this._getAngleCoor(start);
        const /** @type {?} */ endCoor = this._getAngleCoor(end);
        return `M ${endCoor.x} ${endCoor.y} A ${this.radius} ${this.radius} 0 ${largeArc} 0 ${startCoor.x} ${startCoor.y}`;
    }
    /**
     * Get angle coordinates (Cartesian coordinates).
     * @param {?} degrees
     * @return {?}
     */
    _getAngleCoor(degrees) {
        const /** @type {?} */ rads = (degrees - 90) * Math.PI / 180;
        return {
            x: (this.radius * Math.cos(rads)) + this.center,
            y: (this.radius * Math.sin(rads)) + this.center
        };
    }
    /**
     * Calculate/translate the user-defined sectors to arcs.
     * @return {?}
     */
    _calculateSectors() {
        if (!this.sectors) {
            return;
        }
        this._mappedSectors = JSON.parse(JSON.stringify(this.sectors));
        this._mappedSectors.forEach((s) => {
            const /** @type {?} */ ratio = this._end / this.max;
            s.from *= ratio;
            s.to *= ratio;
        });
        this.sectorArcs = this._mappedSectors.map((s) => {
            return {
                path: this._arc(s.from, s.to),
                color: s.color
            };
        });
    }
    /**
     * Update the position of the arrow based on the input.
     * @param {?} input
     * @return {?}
     */
    _updateArrowPos(input) {
        const /** @type {?} */ pos = (this._end / this.max) * input;
        this._renderer.setStyle(this.arrow.nativeElement, 'transform', `rotate(${pos}deg)`);
    }
    /**
     * Rotate the gauge based on the start property. The CSS rotation, saves additional calculations with SVG.
     * @return {?}
     */
    _rotateGauge() {
        const /** @type {?} */ angle = 360 - this.start;
        this._renderer.setStyle(this.gauge.nativeElement, 'transform', `rotate(-${angle}deg)`);
    }
    /**
     * Determine the scale factor (10^n number; i.e. if max = 9000 then scale_factor = 1000)
     * @param {?=} factor
     * @return {?}
     */
    _determineScaleFactor(factor = 10) {
        // Keep smaller factor until 3X
        if (this.max / factor > 30) {
            return this._determineScaleFactor(factor * 10);
        }
        return factor;
    }
    /**
     * Determine the line frequency which represents after what angle we should put a line.
     * @return {?}
     */
    _determineLineFrequency() {
        const /** @type {?} */ separators = this.max / this.scaleFactor;
        const /** @type {?} */ separateAtAngle = this._end / separators;
        let /** @type {?} */ lineFrequency;
        // If separateAtAngle is not an integer, use its value as the line frequency.
        if (separateAtAngle % 1 !== 0) {
            lineFrequency = separateAtAngle;
        }
        else {
            lineFrequency = this.config.INIT_LINE_FREQ * 2;
            for (lineFrequency; lineFrequency <= separateAtAngle; lineFrequency++) {
                if (separateAtAngle % lineFrequency === 0) {
                    break;
                }
            }
        }
        return lineFrequency;
    }
    /**
     * Checks whether the line (based on index) is big or small separator.
     * @param {?} idx
     * @param {?} lineFrequency
     * @return {?}
     */
    _isSeparatorReached(idx, lineFrequency) {
        const /** @type {?} */ separators = this.max / this.scaleFactor;
        const /** @type {?} */ totalSeparators = this._end / lineFrequency;
        const /** @type {?} */ separateAtIdx = totalSeparators / separators;
        if (idx % separateAtIdx === 0) {
            return Separator.Big;
        }
        else if (idx % (separateAtIdx / 2) === 0) {
            return Separator.Small;
        }
        return Separator.NA;
    }
    ;
    /**
     * Creates the scale.
     * @return {?}
     */
    _createScale() {
        const /** @type {?} */ accumWith = this._determineLineFrequency() / 2;
        const /** @type {?} */ isAboveSuitableFactor = this.max / this.scaleFactor > 10;
        let /** @type {?} */ placedVals = 0;
        for (let /** @type {?} */ alpha = 0, /** @type {?} */ i = 0; alpha >= (-1) * this._end; alpha -= accumWith, i++) {
            let /** @type {?} */ lineHeight = this.config.SL_NORM;
            const /** @type {?} */ sepReached = this._isSeparatorReached(i, accumWith);
            // Set the line height based on its type
            switch (sepReached) {
                case Separator.Big:
                    placedVals++;
                    lineHeight = this.config.SL_SEP;
                    break;
                case Separator.Small:
                    lineHeight = this.config.SL_MID_SEP;
                    break;
            }
            // Draw the line
            const /** @type {?} */ higherEnd = this.center - this.config.ARC_STROKE - 2;
            const /** @type {?} */ lowerEnd = higherEnd - lineHeight;
            const /** @type {?} */ alphaRad = Math.PI / 180 * (alpha + 180);
            const /** @type {?} */ sin = Math.sin(alphaRad);
            const /** @type {?} */ cos = Math.cos(alphaRad);
            const /** @type {?} */ color = this._getScaleLineColor(alpha);
            this._addScaleLine(sin, cos, higherEnd, lowerEnd, color);
            // Put a scale value
            if (sepReached === Separator.Big) {
                const /** @type {?} */ isValuePosEven = placedVals % 2 === 0;
                const /** @type {?} */ isLast = alpha <= (-1) * this._end;
                if (!(isAboveSuitableFactor && isValuePosEven && !isLast)) {
                    this._addScaleValue(sin, cos, lowerEnd, alpha);
                }
            }
        }
    }
    /**
     * Get the scale line color from the user-provided sectors definitions.
     * @param {?} alpha
     * @return {?}
     */
    _getScaleLineColor(alpha) {
        alpha *= (-1);
        let /** @type {?} */ color = '';
        if (this._mappedSectors) {
            this._mappedSectors.forEach((s) => {
                if (s.from <= alpha && alpha <= s.to) {
                    color = s.color;
                }
            });
        }
        return color;
    }
    /**
     * Add a scale line to the list that will be later rendered.
     * @param {?} sin
     * @param {?} cos
     * @param {?} higherEnd
     * @param {?} lowerEnd
     * @param {?} color
     * @return {?}
     */
    _addScaleLine(sin, cos, higherEnd, lowerEnd, color) {
        this.scaleLines.push({
            from: {
                x: sin * higherEnd + this.center,
                y: cos * higherEnd + this.center
            },
            to: {
                x: sin * lowerEnd + this.center,
                y: cos * lowerEnd + this.center
            },
            color
        });
    }
    /**
     * Add a scale value.
     * @param {?} sin
     * @param {?} cos
     * @param {?} lowerEnd
     * @param {?} alpha
     * @return {?}
     */
    _addScaleValue(sin, cos, lowerEnd, alpha) {
        let /** @type {?} */ val = Math.round(alpha * (this.max / this._end)) * (-1);
        let /** @type {?} */ posMargin = this.config.TXT_MARGIN * 2;
        // Use the multiplier instead of the real value, if above MAX_PURE_SCALE_VAL (i.e. 1000)
        if (this.max > this.config.MAX_PURE_SCALE_VAL) {
            val /= this.scaleFactor;
            val = Math.round(val * 100) / 100;
            posMargin /= 2;
        }
        this.scaleValues.push({
            text: val.toString(),
            coor: {
                x: sin * (lowerEnd - posMargin) + this.center,
                y: cos * (lowerEnd - posMargin) + this.center
            }
        });
    }
}
GaugeComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-gauge',
                template: `<section class="ng-gauge" [class.light]="lightTheme" [style.width]="size + 'px'">
  <svg class="info" [attr.viewBox]="viewBox" xmlns="http://www.w3.org/2000/svg">
    <circle *ngIf="light"  class="red-light"
      [class.on]="input >= light"
      [attr.cx]="center"
      [attr.cy]="config.LIGHT_Y"
      [attr.r]="config.LIGHT_RADIUS">
    </circle>
    <text *ngIf="max > config.MAX_PURE_SCALE_VAL"
      class="factor"
      [attr.x]="center"
      [attr.y]="config.S_FAC_Y">
      x{{scaleFactor}} {{unit}}
    </text>
    <text *ngIf="showDigital"
      class="digital"
      [attr.x]="center"
      [attr.y]="config.DIGITAL_Y">
      {{input}}
    </text>
    <text class="unit" [attr.x]="center" [attr.y]="config.UNIT_Y">{{unit}}</text>
  </svg>
  <svg #gauge [attr.viewBox]="viewBox" xmlns="http://www.w3.org/2000/svg">
    <path class="main-arc" [attr.d]="arc" [attr.stroke-width]="config.ARC_STROKE" fill="none" />
    <path *ngFor="let arc of sectorArcs"
      [attr.d]="arc.path"
      [attr.stroke]="arc.color"
      [attr.stroke-width]="config.ARC_STROKE"
      fill="none" />
    <line *ngFor="let line of scaleLines"
      [attr.stroke-width]="config.SL_WIDTH"
      [attr.stroke]="line.color || (!lightTheme ? '#333' : '#fff')"
      [attr.x1]="line.from.x"
      [attr.y1]="line.from.y"
      [attr.x2]="line.to.x"
      [attr.y2]="line.to.y" />
    <text *ngFor="let val of scaleValues"
      class="text-val"
      dominant-baseline="central"
      [attr.x]="val.coor.x"
      [attr.y]="val.coor.y"
      [attr.transform]="'rotate(' + gaugeRotationAngle + ', ' + val.coor.x + ', ' + val.coor.y + ')'">
      {{val.text}}
    </text>
    <rect #arrow
      class="arrow"
      [attr.x]="center - config.ARROW_WIDTH / 2"
      [attr.y]="config.ARROW_Y"
      [attr.height]="center - config.ARROW_Y"
      [attr.width]="config.ARROW_WIDTH"
      [attr.rx]="config.ARROW_WIDTH / 2"
      [attr.ry]="config.ARROW_WIDTH / 2">
    </rect>
    <circle class="arrow-pin" [attr.cx]="center" [attr.cy]="center" [attr.r]="config.ARROW_PIN_RAD" />
  </svg>
</section>
`,
                styles: [`@font-face{
  font-family:'Orbitron';
  font-style:normal;
  font-weight:700;
  src:local('Orbitron Bold'), local('Orbitron-Bold'), url(https://fonts.gstatic.com/s/orbitron/v8/Y82YH_MJJWnsH2yUA5AuYY4P5ICox8Kq3LLUNMylGO4.woff2) format('woff2');
  unicode-range:U+0000-00FF, U+0131, U+0152-0153, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2212, U+2215;
}
.ng-gauge{
  position:relative;
  width:400px;
}
.ng-gauge svg.info{
  position:absolute;
  top:0;
  left:0;
}
.ng-gauge rect.arrow{
  -webkit-transform-origin:50% 50%;
          transform-origin:50% 50%;
  fill:orange;
}
.ng-gauge text{
  font-family:'Orbitron', sans-serif;
  font-weight:bold;
  text-anchor:middle;
  fill:#333;
}
.ng-gauge.light text{
  fill:#fff;
}
.ng-gauge text.text-val{
  font-size:12px;
}
.ng-gauge circle.arrow-pin{
  fill:#333;
}
.ng-gauge path.main-arc{
  stroke:#333;
}
.ng-gauge.light path.main-arc{
  stroke:#fff;
}
.ng-gauge text.factor{
  font-size:7px;
}
.ng-gauge text.digital{
  font-size:16px;
}
.ng-gauge text.unit{
  font-size:10px;
}
.ng-gauge circle.red-light{
  fill:#ff4f4f;
  opacity:0.1;
}
.ng-gauge circle.red-light.on{
  opacity:1;
}
`],
                encapsulation: ViewEncapsulation.None
            },] },
];
/** @nocollapse */
GaugeComponent.ctorParameters = () => [
    { type: Renderer2, },
];
GaugeComponent.propDecorators = {
    "gauge": [{ type: ViewChild, args: ['gauge',] },],
    "arrow": [{ type: ViewChild, args: ['arrow',] },],
    "size": [{ type: Input },],
    "start": [{ type: Input },],
    "end": [{ type: Input },],
    "sectors": [{ type: Input },],
    "unit": [{ type: Input },],
    "showDigital": [{ type: Input },],
    "light": [{ type: Input },],
    "lightTheme": [{ type: Input },],
    "factor": [{ type: Input },],
    "config": [{ type: Input },],
    "input": [{ type: Input },],
    "max": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GaugeModule {
}
GaugeModule.decorators = [
    { type: NgModule, args: [{
                imports: [BrowserModule],
                declarations: [GaugeComponent],
                exports: [GaugeComponent]
            },] },
];
/** @nocollapse */
GaugeModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { GaugeComponent, GaugeModule };
//# sourceMappingURL=ng2-gauge.js.map
